@@ -19,7 +19,7 @@
 
 #undef G_LOG_DOMAIN
 #define G_LOG_DOMAIN "e-book-backend-gal"
-#define DEBUG
+#undef DEBUG
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>  
@@ -41,7 +41,7 @@
 #undef LDAP_DEBUG
 #endif
 
-#define d(x) x
+#define d(x)
 
 #include <sys/time.h>
 
@@ -91,17 +91,17 @@ typedef void (*LDAPOpHandler)(LDAPOp *op, LDAPMessage *res);
 typedef void (*LDAPOpDtor)(LDAPOp *op);
 
 struct LDAPOp {
-	LDAPOpHandler handler;
-	LDAPOpDtor    dtor;
-	EBookBackend    *backend;
-	EDataBook       *book;
-	EDataBookView   *view;
-	int            id;
-	guint32 	opid;
+	LDAPOpHandler  handler;
+	LDAPOpDtor     dtor;
+	EBookBackend  *backend;
+	EDataBook     *book;
+	EDataBookView *view;
+	guint32        opid; /* the libebook operation id */
+	int            id;   /* the ldap msg id */
 };
 
 static void     ldap_op_add (LDAPOp *op, EBookBackend *backend, EDataBook *book,
-			     EDataBookView *view, int id, LDAPOpHandler handler, LDAPOpDtor dtor, guint32 opid);
+			     EDataBookView *view, int opid, int msgid, LDAPOpHandler handler, LDAPOpDtor dtor);
 static void     ldap_op_finished (LDAPOp *op);
 
 static gboolean poll_ldap (EBookBackendGAL *bl);
@@ -235,15 +235,13 @@ gal_reconnect (EBookBackendGAL *bl, EDataBookView *book_view, int ldap_status)
 {
 	/* we need to reconnect if we were previously connected */
 	if (bl->priv->connected && ldap_status == LDAP_SERVER_DOWN) {
-		int ldap_error;
-
 		book_view_notify_status (book_view, _("Reconnecting to LDAP server..."));
-		ldap_error = e2k_global_catalog_reconnect (bl->priv->gc,
-							   NULL,
-							   bl->priv->ldap);
+		if (bl->priv->ldap)
+			ldap_unbind (bl->priv->ldap);
+		bl->priv->ldap = e2k_global_catalog_get_ldap (bl->priv->gc, NULL);
 		book_view_notify_status (book_view, "");
 
-		return (ldap_error == LDAP_SUCCESS);
+		return (bl->priv->ldap != NULL);
 	}
 	else {
 		return FALSE;
@@ -253,18 +251,19 @@ gal_reconnect (EBookBackendGAL *bl, EDataBookView *book_view, int ldap_status)
 static void
 ldap_op_add (LDAPOp *op, EBookBackend *backend,
 	     EDataBook *book, EDataBookView *view,
-	     int id,
-	     LDAPOpHandler handler, LDAPOpDtor dtor, guint32 opid)
+	     int opid,
+	     int msgid,
+	     LDAPOpHandler handler, LDAPOpDtor dtor)
 {
 	EBookBackendGAL *bl = E_BOOK_BACKEND_GAL (backend);
 
 	op->backend = backend;
 	op->book = book;
 	op->view = view;
-	op->id = id;
+	op->opid = opid;
+	op->id = msgid;
 	op->handler = handler;
 	op->dtor = dtor;
-	op->opid = opid;
 
 	g_static_rec_mutex_lock (&bl->priv->op_hash_mutex);
 	if (g_hash_table_lookup (bl->priv->id_to_op, &op->id)) {
@@ -329,9 +328,9 @@ ldap_error_to_response (int ldap_error)
 
 static void
 create_contact (EBookBackend *backend,
-				    EDataBook    *book,
-				    guint32       opid,
-				    const char   *vcard)
+		EDataBook    *book,
+		guint32       opid,
+		const char   *vcard)
 {
 	e_data_book_respond_create (book, opid,
 				    GNOME_Evolution_Addressbook_PermissionDenied,
@@ -340,9 +339,9 @@ create_contact (EBookBackend *backend,
 
 static void
 remove_contacts (EBookBackend *backend,
-				     EDataBook    *book,
-				     guint32       opid,
-				     GList        *ids)
+		 EDataBook    *book,
+		 guint32       opid,
+		 GList        *ids)
 {
 	e_data_book_respond_remove_contacts (book, opid,
 					     GNOME_Evolution_Addressbook_PermissionDenied,
@@ -351,9 +350,9 @@ remove_contacts (EBookBackend *backend,
 
 static void
 modify_contact (EBookBackend *backend,
-				    EDataBook    *book,
-                                     guint32      opid,
-				    const char   *vcard)
+		EDataBook    *book,
+		guint32       opid,
+		const char   *vcard)
 {
 	e_data_book_respond_modify (book, opid,
 				    GNOME_Evolution_Addressbook_PermissionDenied,
@@ -383,18 +382,20 @@ get_contact_handler (LDAPOp *op, LDAPMessage *res)
 
 		if (!e) {
 			g_warning ("uh, this shouldn't happen");
-			e_data_book_respond_get_contact (op->book, op->opid,
-						      GNOME_Evolution_Addressbook_OtherError,
-						      "");
+			e_data_book_respond_get_contact (op->book,
+							 op->opid,
+							 GNOME_Evolution_Addressbook_OtherError,
+							 "");
 			ldap_op_finished (op);
 			return;
 		}
 
 		contact = build_contact_from_entry (bl, e, NULL);
 		vcard = e_vcard_to_string (E_VCARD (contact), EVC_FORMAT_VCARD_30);
-		e_data_book_respond_get_contact (op->book, op->opid,
-					      GNOME_Evolution_Addressbook_Success,
-					      vcard);
+		e_data_book_respond_get_contact (op->book,
+						 op->opid,
+						 GNOME_Evolution_Addressbook_Success,
+						 vcard);
 		g_free (vcard);
 		g_object_unref (contact);
 		ldap_op_finished (op);
@@ -411,13 +412,18 @@ get_contact_handler (LDAPOp *op, LDAPMessage *res)
 		}
 		ldap_memfree (ldap_error_msg);
 
-		e_data_book_respond_get_contact (op->book, op->opid, ldap_error_to_response (ldap_error), "");
+		e_data_book_respond_get_contact (op->book,
+						 op->opid,
+						 ldap_error_to_response (ldap_error),
+						 "");
 		ldap_op_finished (op);
 	}
 	else {
 		g_warning ("unhandled result type %d returned", msg_type);
-		e_data_book_respond_get_contact (op->book, op->opid, GNOME_Evolution_Addressbook_OtherError,
-					    "");
+		e_data_book_respond_get_contact (op->book,
+						 op->opid,
+						 GNOME_Evolution_Addressbook_OtherError,
+						 "");
 		ldap_op_finished (op);
 	}
 
@@ -439,7 +445,6 @@ get_contact (EBookBackend *backend,
 {
 	LDAPGetContactOp *get_contact_op = g_new0 (LDAPGetContactOp, 1);
 	EBookBackendGAL *bl = E_BOOK_BACKEND_GAL (backend);
-	LDAP *ldap = bl->priv->ldap;
 	int get_contact_msgid;
 	EDataBookView *book_view;
 	int ldap_error;
@@ -447,7 +452,7 @@ get_contact (EBookBackend *backend,
 	book_view = find_book_view (bl);
 
 	do {	
-		ldap_error = ldap_search_ext (ldap, id,
+		ldap_error = ldap_search_ext (bl->priv->ldap, id,
 					      LDAP_SCOPE_BASE,
 					      "(objectclass=*)",
 					      search_attrs, 0, NULL, NULL,
@@ -457,13 +462,14 @@ get_contact (EBookBackend *backend,
 
 	if (ldap_error == LDAP_SUCCESS) {
 		ldap_op_add ((LDAPOp*)get_contact_op, backend, book,
-			     book_view, get_contact_msgid,
-			     get_contact_handler, get_contact_dtor, opid);
+			     book_view, opid, get_contact_msgid,
+			     get_contact_handler, get_contact_dtor);
 	}
 	else {
-		e_data_book_respond_get_contact (book, opid,
-					    ldap_error_to_response (ldap_error),
-					    "");
+		e_data_book_respond_get_contact (book,
+						 opid,
+						 ldap_error_to_response (ldap_error),
+						 "");
 		get_contact_dtor ((LDAPOp*)get_contact_op);
 	}
 }
@@ -491,7 +497,7 @@ contact_list_handler (LDAPOp *op, LDAPMessage *res)
 			EContact *contact = build_contact_from_entry (bl, e, NULL);
 			char *vcard = e_vcard_to_string (E_VCARD (contact), EVC_FORMAT_VCARD_30);
 
-			printf ("vcard = %s\n", vcard);
+			d(printf ("vcard = %s\n", vcard));
  
 			contact_list_op->contacts = g_list_append (contact_list_op->contacts,
 								   vcard);
@@ -513,22 +519,26 @@ contact_list_handler (LDAPOp *op, LDAPMessage *res)
 		}
 		ldap_memfree (ldap_error_msg);
 
-		g_warning ("search returned %d\n", ldap_error);
+		d(printf ("search returned %d\n", ldap_error));
 
 		if (ldap_error == LDAP_TIMELIMIT_EXCEEDED)
-			e_data_book_respond_get_contact_list (op->book, op->opid,
+			e_data_book_respond_get_contact_list (op->book,
+							      op->opid,
 							      GNOME_Evolution_Addressbook_SearchTimeLimitExceeded,
 							      contact_list_op->contacts);
 		else if (ldap_error == LDAP_SIZELIMIT_EXCEEDED)
-			e_data_book_respond_get_contact_list (op->book, op->opid,
+			e_data_book_respond_get_contact_list (op->book,
+							      op->opid,
 							      GNOME_Evolution_Addressbook_SearchSizeLimitExceeded,
 							      contact_list_op->contacts);
 		else if (ldap_error == LDAP_SUCCESS)
-			e_data_book_respond_get_contact_list (op->book, op->opid,
+			e_data_book_respond_get_contact_list (op->book,
+							      op->opid,
 							      GNOME_Evolution_Addressbook_Success,
 							      contact_list_op->contacts);
 		else
-			e_data_book_respond_get_contact_list (op->book, op->opid,
+			e_data_book_respond_get_contact_list (op->book,
+							      op->opid,
 							      GNOME_Evolution_Addressbook_OtherError,
 							      contact_list_op->contacts);
 
@@ -536,7 +546,8 @@ contact_list_handler (LDAPOp *op, LDAPMessage *res)
 	}
 	else {
 		g_warning ("unhandled search result type %d returned", msg_type);
-		e_data_book_respond_get_contact_list (op->book, op->opid,
+		e_data_book_respond_get_contact_list (op->book,
+						      op->opid,
 						      GNOME_Evolution_Addressbook_OtherError,
 						      NULL);
 		ldap_op_finished (op);
@@ -562,7 +573,6 @@ get_contact_list (EBookBackend *backend,
 	LDAPGetContactListOp *contact_list_op = g_new0 (LDAPGetContactListOp, 1);
 	EBookBackendGAL *bl = E_BOOK_BACKEND_GAL (backend);
 	GNOME_Evolution_Addressbook_CallStatus status;
-	LDAP *ldap = bl->priv->ldap;
 	int contact_list_msgid;
 	EDataBookView *book_view;
 	int ldap_error;
@@ -576,10 +586,10 @@ get_contact_list (EBookBackend *backend,
 		return;
 	}
 
-	printf ("getting contact list with filter: %s\n", ldap_query);
+	d(printf ("getting contact list with filter: %s\n", ldap_query));
 
 	do {	
-		ldap_error = ldap_search_ext (ldap, LDAP_ROOT_DSE,
+		ldap_error = ldap_search_ext (bl->priv->ldap, LDAP_ROOT_DSE,
 					      LDAP_SCOPE_SUBTREE,
 					      ldap_query,
 					      search_attrs, 0, NULL, NULL,
@@ -591,11 +601,12 @@ get_contact_list (EBookBackend *backend,
 
 	if (ldap_error == LDAP_SUCCESS) {
 		ldap_op_add ((LDAPOp*)contact_list_op, backend, book,
-			     book_view, contact_list_msgid,
-			     contact_list_handler, contact_list_dtor, opid);
+			     book_view, opid, contact_list_msgid,
+			     contact_list_handler, contact_list_dtor);
 	}
 	else {
-		e_data_book_respond_get_contact_list (book, opid,
+		e_data_book_respond_get_contact_list (book,
+						      opid,
 						      ldap_error_to_response (ldap_error),
 						      NULL);
 		contact_list_dtor ((LDAPOp*)contact_list_op);
@@ -911,8 +922,17 @@ build_query (EBookBackendGAL *bl, const char *query, char **ldap_query)
 	r = e_sexp_eval (sexp);
 
 	if (r->type == ESEXP_RES_STRING) {
-		*ldap_query = g_strdup_printf ("(&(mail=*)(!(msExchHideFromAddressLists=TRUE))%s)", r->value.string);
-		retval = GNOME_Evolution_Addressbook_Success;
+		if (!strcmp (r->value.string, "(mail=*)")) {
+			/* If the query is empty, 
+			 * don't search for all the contats 
+			 */ 
+			*ldap_query = NULL;
+			retval = GNOME_Evolution_Addressbook_QueryRefused;
+		}
+		else {
+			*ldap_query = g_strdup_printf ("(&(mail=*)(!(msExchHideFromAddressLists=TRUE))%s)", r->value.string);
+			retval = GNOME_Evolution_Addressbook_Success;
+		}
 	} else if (r->type == ESEXP_RES_BOOL) {
 		/* If it's FALSE, that means "no matches". If it's TRUE
 		 * that means "everything matches", but we don't support
@@ -994,8 +1014,8 @@ build_contact_from_entry (EBookBackendGAL *bl, LDAPMessage *e, GList **existing_
 					break;
 				}
 
-			printf ("attr = %s, ", attr);
-			printf ("info = %p\n", info);
+			d(printf ("attr = %s, ", attr));
+			d(printf ("info = %p\n", info));
 
 			if (info) {
 				if (1) {
@@ -1003,7 +1023,7 @@ build_contact_from_entry (EBookBackendGAL *bl, LDAPMessage *e, GList **existing_
 
 					if (values) {
 						if (info->prop_type & PROP_TYPE_STRING) {
-							printf ("value = %s\n", values[0]);
+							d(printf ("value = %s\n", values[0]));
 							/* if it's a normal property just set the string */
 							if (values[0])
 								e_contact_set (contact, info->field_id, values[0]);
@@ -1049,7 +1069,7 @@ poll_ldap (EBookBackendGAL *bl)
 	if (rc != 0) {/* rc == 0 means timeout exceeded */
 		if (rc == -1) {
 			EDataBookView *book_view = find_book_view (bl);
-			g_warning ("ldap_result returned -1, restarting ops");
+			d(printf ("ldap_result returned -1, restarting ops\n"));
 
 			gal_reconnect (bl, book_view, LDAP_SERVER_DOWN);
 #if 0
@@ -1166,10 +1186,19 @@ start_book_view (EBookBackend  *backend,
 {
 	EBookBackendGAL *bl = E_BOOK_BACKEND_GAL (backend);
 	GNOME_Evolution_Addressbook_CallStatus status;
-	LDAP *ldap = bl->priv->ldap;
 	char *ldap_query;
 	int ldap_err;
 	int search_msgid;
+	int view_limit;
+
+	/* we start at 1 so the user sees stuff as it appears and we
+	   aren't left waiting for more cards to show up, if the
+	   connection is slow. */
+	e_data_book_view_set_thresholds (view, 1, 3000);
+
+	view_limit = e_data_book_view_get_max_results (view);
+	if (view_limit == -1 || view_limit > bl->priv->gc->response_limit)
+		view_limit = bl->priv->gc->response_limit;
 
 	d(printf ("start_book_view (%p)\n", view));
 
@@ -1183,14 +1212,14 @@ start_book_view (EBookBackend  *backend,
 	do {
 		book_view_notify_status (view, _("Searching..."));
 
-		ldap_err = ldap_search_ext (ldap, LDAP_ROOT_DSE,
+		ldap_err = ldap_search_ext (bl->priv->ldap, LDAP_ROOT_DSE,
 					    LDAP_SCOPE_SUBTREE,
 					    ldap_query,
 					    search_attrs, 0,
 					    NULL, /* XXX */
 					    NULL, /* XXX */
 					    NULL, /* XXX timeout */
-					    e_data_book_view_get_max_results (view),
+					    view_limit,
 					    &search_msgid);
 	} while (gal_reconnect (bl, view, ldap_err));
 
@@ -1215,8 +1244,8 @@ start_book_view (EBookBackend  *backend,
 		bonobo_object_ref (view);
 
 		ldap_op_add ((LDAPOp*)op, E_BOOK_BACKEND (bl), NULL, view,
-			     search_msgid,
-			     ldap_search_handler, ldap_search_dtor, 0);
+			     0, search_msgid,
+			     ldap_search_handler, ldap_search_dtor);
 
 		g_object_set_data (G_OBJECT (view), "EBookBackendGAL.BookView::search_op", op);
 	}
@@ -1224,7 +1253,7 @@ start_book_view (EBookBackend  *backend,
 
 static void
 stop_book_view (EBookBackend  *backend,
-				    EDataBookView *view)
+		EDataBookView *view)
 {
 	LDAPSearchOp *op;
 
@@ -1253,7 +1282,8 @@ authenticate_user (EBookBackend *backend,
 		   const char   *auth_method)
 {
 	/* We should not be here */
-	e_data_book_respond_authenticate_user (book, opid,
+	e_data_book_respond_authenticate_user (book,
+					       opid,
 					       GNOME_Evolution_Addressbook_UnsupportedAuthenticationMethod);
 }
 
@@ -1263,7 +1293,8 @@ get_supported_fields (EBookBackend *backend,
 		      guint32 	    opid)
 
 {
-	e_data_book_respond_get_supported_fields (book, opid,
+	e_data_book_respond_get_supported_fields (book,
+						  opid,
 						  GNOME_Evolution_Addressbook_Success,
 						  supported_fields);
 }
@@ -1274,11 +1305,34 @@ get_supported_auth_methods (EBookBackend *backend,
 			    guint32       opid)
 
 {
-	e_data_book_respond_get_supported_auth_methods (book, opid,
+	e_data_book_respond_get_supported_auth_methods (book,
+							opid,
 							GNOME_Evolution_Addressbook_Success,
 							NULL);
 }
 
+static void
+ldap_cancel_op(void *key, void *value, void *data)
+{
+	EBookBackendGAL *bl = data;
+	LDAPOp *op = value;
+
+	/* ignore errors, its only best effort? */
+	ldap_abandon_ext (bl->priv->ldap, op->id, NULL, NULL);
+}
+
+static GNOME_Evolution_Addressbook_CallStatus
+cancel_operation (EBookBackend *backend, EDataBook *book)
+{
+	EBookBackendGAL *bl = E_BOOK_BACKEND_GAL (backend);
+
+	g_static_rec_mutex_lock (&bl->priv->op_hash_mutex);
+	g_hash_table_foreach (bl->priv->id_to_op, ldap_cancel_op, bl);
+	g_static_rec_mutex_unlock (&bl->priv->op_hash_mutex);
+
+	return GNOME_Evolution_Addressbook_Success;
+}
+
 static GNOME_Evolution_Addressbook_CallStatus
 load_source (EBookBackend *backend,
 	     ESource      *source,
@@ -1314,6 +1368,10 @@ get_static_capabilities (EBookBackend *backend)
 
 /**
  * e_book_backend_gal_new:
+ *
+ * Creates a new #EBookBackendGAL.
+ *
+ * Return value: the new #EBookBackendGAL.
  */
 EBookBackend *
 e_book_backend_gal_new (void)
@@ -1353,7 +1411,7 @@ dispose (GObject *object)
 		g_static_rec_mutex_free (&bl->priv->op_hash_mutex);
 
 		if (bl->priv->poll_timeout != -1) {
-			printf ("removing timeout\n");
+			d(printf ("removing timeout\n"));
 			g_source_remove (bl->priv->poll_timeout);
 		}
 
@@ -1395,6 +1453,7 @@ class_init (EBookBackendGALClass *klass)
 	backend_class->authenticate_user          = authenticate_user;
 	backend_class->get_supported_fields       = get_supported_fields;
 	backend_class->get_supported_auth_methods = get_supported_auth_methods;
+	backend_class->cancel_operation           = cancel_operation;
 
 	object_class->dispose = dispose;
 
