@@ -27,6 +27,8 @@
 #include <string.h>
 #include <sys/stat.h>
 
+#include <camel/camel-i18n.h>
+
 #include "camel-exchange-folder.h"
 #include "camel-exchange-search.h"
 #include "camel-exchange-store.h"
@@ -626,7 +628,7 @@ transfer_messages_to (CamelFolder *source, GPtrArray *uids,
 {
 	CamelExchangeFolder *exch_source = CAMEL_EXCHANGE_FOLDER (source);
 	CamelExchangeFolder *exch_dest = CAMEL_EXCHANGE_FOLDER (dest);
-	GPtrArray *ret_uids;
+	GPtrArray *ret_uids = NULL;
 	int hier_len;
 
 	camel_operation_start (NULL, delete_originals ? _("Moving messages") :
@@ -649,7 +651,9 @@ transfer_messages_to (CamelFolder *source, GPtrArray *uids,
 			     CAMEL_STUB_ARG_RETURN,
 			     CAMEL_STUB_ARG_STRINGARRAY, &ret_uids,
 			     CAMEL_STUB_ARG_END)) {
-		cache_xfer (exch_source, exch_dest, uids, ret_uids, FALSE);
+		if (ret_uids->len != 0)
+			cache_xfer (exch_source, exch_dest, uids, ret_uids, FALSE);
+
 		if (transferred_uids)
 			*transferred_uids = ret_uids;
 		else {
@@ -696,6 +700,17 @@ find_parent (CamelExchangeFolder *exch, const char *thread_index)
 	return msgid;
 }
 
+/**
+ * camel_exchange_folder_add_message:
+ * @exch: the folder
+ * @uid: UID of the new message
+ * @flags: message flags
+ * @size: size of the new message
+ * @headers: RFC822 headers of the new message
+ *
+ * Append the new message described by @uid, @flags, @size, and
+ * @headers to @exch's summary.
+ **/
 void
 camel_exchange_folder_add_message (CamelExchangeFolder *exch,
 				   const char *uid, guint32 flags,
@@ -756,6 +771,13 @@ camel_exchange_folder_add_message (CamelExchangeFolder *exch,
 	return;
 }
 
+/**
+ * camel_exchange_folder_remove_message:
+ * @exch: the folder
+ * @uid: message to remove
+ *
+ * Remove the message indicated by @uid from @exch's summary and cache.
+ **/
 void
 camel_exchange_folder_remove_message (CamelExchangeFolder *exch,
 				      const char *uid)
@@ -793,6 +815,14 @@ camel_exchange_folder_remove_message (CamelExchangeFolder *exch,
 	camel_folder_change_info_free (changes);
 }
 
+/**
+ * camel_exchange_folder_uncache_message:
+ * @exch: the folder
+ * @uid: message to uncache
+ *
+ * Remove the message indicated by @uid from @exch's cache, but NOT
+ * from the summary.
+ **/
 void
 camel_exchange_folder_uncache_message (CamelExchangeFolder *exch,
 				       const char *uid)
@@ -800,6 +830,15 @@ camel_exchange_folder_uncache_message (CamelExchangeFolder *exch,
 	camel_data_cache_remove (exch->cache, "cache", uid, NULL);
 }
 
+/**
+ * camel_exchange_folder_update_message_flags:
+ * @exch: the folder
+ * @uid: the message
+ * @flags: new message flags
+ *
+ * Update the message flags of @uid in @exch's summary to be @flags.
+ * Only the bits in the mask %CAMEL_EXCHANGE_SERVER_FLAGS are valid.
+ **/
 void
 camel_exchange_folder_update_message_flags (CamelExchangeFolder *exch,
 					    const char *uid, guint32 flags)
@@ -826,6 +865,16 @@ camel_exchange_folder_update_message_flags (CamelExchangeFolder *exch,
 	}
 }
 
+/**
+ * camel_exchange_folder_update_message_tag:
+ * @exch: the folder
+ * @uid: the message
+ * @name: the tag name
+ * @value: the new value for @name
+ *
+ * Update the value of tag @name of @uid in @exch's summary to be
+ * @value (or remove the tag altogether if @value is %NULL).
+ **/
 void
 camel_exchange_folder_update_message_tag (CamelExchangeFolder *exch,
 					  const char *uid,
@@ -850,6 +899,21 @@ camel_exchange_folder_update_message_tag (CamelExchangeFolder *exch,
 	camel_folder_change_info_free (changes);
 }
 
+/**
+ * camel_exchange_folder_construct:
+ * @folder: the folder
+ * @parent: @folder's parent store
+ * @name: the full name of the folder
+ * @camel_flags: the folder flags passed to camel_store_get_folder().
+ * @folder_dir: local directory this folder can cache data into
+ * @stub: the #CamelStub.
+ * @ex: a #CamelException
+ *
+ * Constructs @folder by requesting the necessary data from the server
+ * via @stub.
+ *
+ * Return value: success or failure.
+ **/
 gboolean
 camel_exchange_folder_construct (CamelFolder *folder, CamelStore *parent,
 				 const char *name, guint32 camel_flags,
