@@ -44,6 +44,7 @@ struct _EFolderExchangePrivate {
 	char *internal_uri, *permanent_uri;
 	char *outlook_class, *storage_dir;
 	const char *path;
+	long long int folder_size;
 	gboolean has_subfolders;
 };
 
@@ -121,6 +122,9 @@ e_folder_exchange_new (ExchangeHierarchy *hier, const char *name,
 {
 	EFolderExchange *efe;
 	EFolder *ef;
+	char *conf_key_cal="/apps/evolution/calendar/sources";
+	char *conf_key_tasks="/apps/evolution/tasks/sources";
+	char *conf_key_contacts="/apps/evolution/addressbook/sources";
 	ESourceList *cal_source_list, *task_source_list, *cont_source_list;
 
 	g_return_val_if_fail (EXCHANGE_IS_HIERARCHY (hier), NULL);
@@ -149,12 +153,13 @@ e_folder_exchange_new (ExchangeHierarchy *hier, const char *name,
 		    (strcmp (type, "calendar/public") == 0)) {
 			cal_source_list = e_source_list_new_for_gconf ( 
 						gconf_client_get_default (), 
-						CONF_KEY_CAL);
+						conf_key_cal);
 			add_esource (hier->account, 
-				     EXCHANGE_CALENDAR_FOLDER, 
+				     conf_key_cal, 
 				     name, 
 				     physical_uri, 
-				     &cal_source_list);
+				     &cal_source_list,
+				     FALSE);
 			e_source_list_sync (cal_source_list, NULL);
 			g_object_unref (cal_source_list);
 		}
@@ -162,12 +167,13 @@ e_folder_exchange_new (ExchangeHierarchy *hier, const char *name,
 			 (strcmp (type, "tasks/public") == 0)) {
 			task_source_list = e_source_list_new_for_gconf ( 
 						gconf_client_get_default (), 
-						CONF_KEY_TASKS);
+						conf_key_tasks);
 			add_esource (hier->account, 
-				     EXCHANGE_TASKS_FOLDER, 
+				     conf_key_tasks, 
 				     name, 
 				     physical_uri, 
-				     &task_source_list);
+				     &task_source_list,
+				     FALSE);
 			e_source_list_sync (task_source_list, NULL);
 			g_object_unref (task_source_list);
 		}
@@ -175,12 +181,13 @@ e_folder_exchange_new (ExchangeHierarchy *hier, const char *name,
 			 (strcmp (type, "contacts/public") == 0)) {
 			cont_source_list = e_source_list_new_for_gconf ( 
 						gconf_client_get_default (), 
-						CONF_KEY_CONTACTS);
+						conf_key_contacts);
 			add_esource (hier->account, 
-				     EXCHANGE_CONTACTS_FOLDER, 
+				     conf_key_contacts, 
 				     name, 
 				     physical_uri, 
-				     &cont_source_list);
+				     &cont_source_list,
+				     TRUE);
 			e_source_list_sync (cont_source_list, NULL);
 			g_object_unref (cont_source_list);
 		}
@@ -280,6 +287,43 @@ e_folder_exchange_set_permanent_uri (EFolder *folder, const char *permanent_uri)
 	efe->priv->permanent_uri = g_strdup (permanent_uri);
 }
 
+/**
+ * e_folder_exchange_get_folder_size:
+ * @folder: an #EFolderExchange
+ *
+ * Returns the folder's size. See docs/entryids for more
+ * details.
+ *
+ * Return value: @folder's size
+ **/
+long long int
+e_folder_exchange_get_folder_size (EFolder *folder)
+{
+	g_return_val_if_fail (E_IS_FOLDER_EXCHANGE (folder), NULL);
+
+	return E_FOLDER_EXCHANGE (folder)->priv->folder_size;
+}
+
+/**
+ * e_folder_exchange_set_folder_size:
+ * @folder: an #EFolderExchange
+ * @folder_size: folder size
+ *
+ * Sets @folder's folder_size
+ **/
+void
+e_folder_exchange_set_folder_size (EFolder *folder, long long int folder_size)
+{
+	EFolderExchange *efe;
+
+	g_return_if_fail (E_IS_FOLDER_EXCHANGE (folder));
+
+	efe = E_FOLDER_EXCHANGE (folder);
+
+	efe->priv->folder_size = folder_size;
+}
+
+
 /**
  * e_folder_exchange_get_has_subfolders:
  * @folder: an #EFolderExchange
@@ -386,6 +430,7 @@ e_folder_exchange_save_to_file (EFolder *folder, const char *filename)
 	xmlNode *root;
 	const char *name, *type, *outlook_class;
 	const char *physical_uri, *internal_uri, *permanent_uri;
+	const char *folder_size;
 	int status;
 
 	name = e_folder_get_name (folder);
@@ -394,6 +439,7 @@ e_folder_exchange_save_to_file (EFolder *folder, const char *filename)
 	physical_uri = e_folder_get_physical_uri (folder);
 	internal_uri = e_folder_exchange_get_internal_uri (folder);
 	permanent_uri = e_folder_exchange_get_permanent_uri (folder);
+	folder_size = g_strdup_printf ("%d", e_folder_exchange_get_folder_size (folder));
 
 	g_return_val_if_fail (name && type && physical_uri && internal_uri,
 			      FALSE);
@@ -408,6 +454,7 @@ e_folder_exchange_save_to_file (EFolder *folder, const char *filename)
 	xmlNewChild (root, NULL, "outlook_class", outlook_class);
 	xmlNewChild (root, NULL, "physical_uri", physical_uri);
 	xmlNewChild (root, NULL, "internal_uri", internal_uri);
+	xmlNewChild (root, NULL, "folder_size", folder_size);
 	if (permanent_uri)
 		xmlNewChild (root, NULL, "permanent_uri", permanent_uri);
 
@@ -438,6 +485,7 @@ e_folder_exchange_new_from_file (ExchangeHierarchy *hier, const char *filename)
 	char *type = NULL, *outlook_class = NULL;
 	char *physical_uri = NULL, *internal_uri = NULL;
 	char *permanent_uri = NULL;
+	char *folder_size = NULL;
 
 	doc = xmlParseFile (filename);
 	if (!doc)
@@ -493,6 +541,12 @@ e_folder_exchange_new_from_file (ExchangeHierarchy *hier, const char *filename)
 		e_folder_exchange_set_permanent_uri (folder, permanent_uri);
 	}
 
+	node = e_xml_get_child_by_name (root, "folder_size");
+	if (node) {
+		folder_size = xmlNodeGetContent (node);
+		e_folder_exchange_set_folder_size (folder, atoi (folder_size));
+	}
+
  done:
 	xmlFree (display_name);
 	xmlFree (type);
@@ -808,9 +862,14 @@ E2kHTTPStatus
 e_folder_exchange_delete (EFolder *folder, E2kOperation *op)
 {
 	ExchangeHierarchy *hier;
+	char *conf_key_cal="/apps/evolution/calendar/sources";
+	char *conf_key_tasks="/apps/evolution/tasks/sources";
+	char *conf_key_contacts="/apps/evolution/addressbook/sources";
 	ESourceList *cal_source_list, *task_source_list, *cont_source_list;
 	const char *folder_type, *physical_uri;
 
+	g_return_val_if_fail (E_IS_FOLDER_EXCHANGE (folder), NULL);
+
 	/* remove ESources */
 	hier = e_folder_exchange_get_hierarchy (folder); 
 
@@ -823,12 +882,12 @@ e_folder_exchange_delete (EFolder *folder, E2kOperation *op)
 		    (strcmp (folder_type, "calendar/public") == 0)) {
 			cal_source_list = e_source_list_new_for_gconf (
 						gconf_client_get_default (),
-						CONF_KEY_CAL);
+						conf_key_cal);
 			remove_esource (hier->account, 
-					EXCHANGE_CALENDAR_FOLDER,
+					conf_key_cal,
 					physical_uri,
 					&cal_source_list,
-					FALSE);
+					FALSE, FALSE);
 			e_source_list_sync (cal_source_list, NULL);
 			g_object_unref (cal_source_list);
 		}
@@ -836,12 +895,12 @@ e_folder_exchange_delete (EFolder *folder, E2kOperation *op)
 			 (strcmp (folder_type, "tasks/public") == 0)) {
 			task_source_list = e_source_list_new_for_gconf (
 						gconf_client_get_default (),
-						CONF_KEY_TASKS);
+						conf_key_tasks);
 			remove_esource (hier->account,
-					EXCHANGE_TASKS_FOLDER,
+					conf_key_tasks,
 					physical_uri,
 					&task_source_list,
-					FALSE);
+					FALSE, FALSE);
 			e_source_list_sync (task_source_list, NULL);
 			g_object_unref (task_source_list);
 		}
@@ -849,12 +908,12 @@ e_folder_exchange_delete (EFolder *folder, E2kOperation *op)
 			 (strcmp (folder_type, "contacts/public") == 0)) { 
 			cont_source_list = e_source_list_new_for_gconf (
 						gconf_client_get_default (),
-						CONF_KEY_CONTACTS);
+						conf_key_contacts);
 			remove_esource (hier->account,
-					EXCHANGE_CONTACTS_FOLDER,
+					conf_key_contacts,
 					physical_uri,
 					&cont_source_list,
-					FALSE);
+					FALSE, TRUE);
 			e_source_list_sync (cont_source_list, NULL);
 			g_object_unref (cont_source_list);
 		}
