@@ -50,6 +50,8 @@
 #include <stdlib.h>
 #include <string.h>
 
+#define d(x) x
+
 struct _ExchangeAccountPrivate {
 	E2kContext *ctx;
 	E2kGlobalCatalog *gc;
@@ -66,7 +68,7 @@ struct _ExchangeAccountPrivate {
 	gboolean uris_use_email;
 
 	char *identity_name, *identity_email, *source_uri, *password_key;
-	char *username, *password, *windows_domain, *ad_server;
+	char *username, *password, *ad_server;
 	E2kAutoconfigAuthPref auth_pref;
 	int ad_limit;
 
@@ -265,9 +267,6 @@ finalize (GObject *object)
 			strlen (account->priv->password));
 		g_free (account->priv->password);
 	}
-	if (account->priv->windows_domain)
-		g_free (account->priv->windows_domain);
-
 	if (account->priv->ad_server)
 		g_free (account->priv->ad_server);
 
@@ -962,7 +961,12 @@ exchange_account_connect (ExchangeAccount *account)
 	struct dirent *dent;
 	DIR *d;
 	char *old_password, *new_password;
-	char *user_name = NULL;
+	E2kGlobalCatalogStatus gcstatus;
+	E2kGlobalCatalogEntry *entry;
+	E2kOperation gcop;
+	const char *quota_msg = NULL;
+	GtkWidget *quota_dialog;
+	int quota_resp;
 
 	g_return_val_if_fail (EXCHANGE_IS_ACCOUNT (account), NULL);
 
@@ -979,18 +983,10 @@ exchange_account_connect (ExchangeAccount *account)
 	account->priv->connecting = TRUE;
 	g_mutex_unlock (account->priv->connect_lock);
 
-	if (account->priv->windows_domain)
-		user_name = g_strdup_printf ("%s\\%s", account->priv->windows_domain, 
-							account->priv->username);
-	else
-		user_name = g_strdup (account->priv->username);
-
 	ac = e2k_autoconfig_new (account->home_uri,
-				 user_name,
+				 account->priv->username,
 				 NULL,
 				 account->priv->auth_pref);
-	g_free (user_name);
-
 	e2k_autoconfig_set_gc_server (ac, account->priv->ad_server,
 				      account->priv->ad_limit);
 
@@ -1150,6 +1146,27 @@ exchange_account_connect (ExchangeAccount *account)
 		if (timezone)
 			account->default_timezone = g_strdup (timezone);
 	}
+	/* Check for quota warnings */
+	e2k_operation_init (&gcop);
+	gcstatus = e2k_global_catalog_lookup (account->priv->gc, &gcop,
+                                            E2K_GLOBAL_CATALOG_LOOKUP_BY_EMAIL,
+                                            account->priv->identity_email,
+					    E2K_GLOBAL_CATALOG_LOOKUP_QUOTA,
+                                            &entry);	
+	e2k_operation_free (&gcop);
+	
+	if (gcstatus == E2K_GLOBAL_CATALOG_OK) {
+
+		if (entry->quota_norecv) {
+			quota_msg = g_strdup_printf ("You have exceeded your quota for storing mails on this server. Your current usage is : %d . You will not be able to either send or recieve mails now\n", entry->quota_norecv);
+		} else if (entry->quota_nosend) {
+			quota_msg = g_strdup_printf ("You are nearing your quota available for storing mails on this server. Your current usage is : %d . You will not be able to send mails till you clear up some space by deleting some mails.\n", entry->quota_nosend);
+		} else if (entry->quota_warn) {
+			quota_msg = g_strdup_printf ("You are nearing your quota available for storing mails on this server. Your current usage is : %d . Try to clear up some space by deleting some mails.\n", entry->quota_warn);
+		}
+		
+		e_notice (NULL, GTK_MESSAGE_INFO, quota_msg);
+	}
 
 	/* Set up Personal Folders hierarchy */
 	phys_uri_prefix = g_strdup_printf ("exchange://%s/personal",
@@ -1520,12 +1537,6 @@ exchange_account_new (EAccountList *account_list, EAccount *adata)
 	account->priv->password_key = g_strdup_printf ("exchange://%s", account->priv->uri_authority);
 
 	account->priv->username = g_strdup (uri->user);
-
-	if (uri->domain)
-		account->priv->windows_domain = g_strdup (uri->domain);
-	else
-		account->priv->windows_domain = NULL;
-
 	account->exchange_server = g_strdup (uri->host);
 	if (uri->authmech && !strcmp (uri->authmech, "Basic"))
 		account->priv->auth_pref = E2K_AUTOCONFIG_USE_BASIC;
